package models

import (
	"database/sql"
	"time"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
	"xorm.io/xorm"
)

func (event *Event) Insert(cfg *config.Config) (err error) {
	session := cfg.Db.NewSession()
	defer session.Close()
	_, err = session.Insert(event)
	err = session.Commit()
	return
}

func (event *Event) APIFormat() *APIEvent {
	if event == nil {
		return nil
	}
	return &APIEvent{
		IP:       event.IP.IP,
		Src:      event.Src,
		Hostname: event.Hostname.String,
	}
}

func (event *Event) APIParse(session *xorm.Session, apievent APIEvent) (err error) {
	*event = Event{
		IP:  &IP{IP: apievent.IP},
		Src: apievent.Src,
		Hostname: sql.NullString{
			String: apievent.Hostname,
			Valid:  true},
	}
	event.IP.GetOrCreate(session)
	return
}

type Event struct {
	ID       int            `xorm:"pk autoincr"`
	Src      string         `xorm:"text notnull"`
	Hostname sql.NullString `xorm:"text default NULL"`
	Mode     sql.NullString `xorm:"text default NULL"`
	IP       *IP            `xorm:"int ip_id"`
	Created  time.Time      `xorm:"created notnull"`
}

type APIEvent struct {
	IP       string `json:"ip"`
	Src      string `json:"src"`
	Hostname string `json:"hostname"`
	Mode     string `json:"mode"`
	Created  string `json:"created"`
}
